/*
Java Graphics Demos: 
by Orion Lawlor,  1/1999
olawlor@acm.org

This code is totally free, and
should be considered in the public domain.
Use and modify it at will.
*/

import java.awt.*;
import java.awt.image.*;
import java.util.*;

/*
My "Just the bytes, ma'am" 2D image class.

I use this class to quickly and easily
get at the pixels of an RGB image.
*/
class image
{
	public int w,h;
	public int scan;
	public int pix[];
	
	public void init(int Npix[],int Nw,int Nh,int Nscan)
	{
		pix=Npix;w=Nw;h=Nh;
		if (Nscan!=0)
			scan=Nscan;
		else
			scan=w;
	}
	//Return a deep copy of this image
	public image dup()
	{
		int newPix[]=new int[scan*h];
		int i;
		for (i=0;i<scan*h;i++)
			newPix[i]=pix[i];
		image newImage=new image();
		newImage.init(newPix,w,h,scan);
		return newImage;
	}
	public Rectangle getRect(int topleftX,int topleftY)
	{
		return new Rectangle(topleftX,topleftY,w,h);
	}
	//Fills this image with the given solid color,
	//by writing the color to each pixel.
	public void fill(int color)
	{
		int x,y,l;
		for (y=0;y<h;y++)
		{
			l=y*scan;
			for (x=0;x<w;x++)
				pix[l+x]=color;
		}
	}
	//AndBy performs a bitwise "and" of each pixel 
	//of this image with the given bit mask.
	public void andBy(int bitMask)
	{
		int x,y,l;
		for (y=0;y<h;y++)
		{
			l=y*scan;
			for (x=0;x<w;x++)
				pix[l+x]&=bitMask;
		}
	}
	
}

/*
Image2image is an adapter class that
translates a Java AWT "Image" into my 
"image".  This class is necessary because
Java "Image"s are so darn complex-- getting
to the actual RGB bytes in an "Image" is 
tough and slow.
*/
class Image2image extends image implements ImageConsumer
{
	boolean imageDone=false,error=false;
/*ImageConsumer internals (must be public; shouldn't be called)*/
	public void imageComplete(int status)
	{
		if (status==ImageConsumer.STATICIMAGEDONE)
			imageDone=true;
		else if (status==ImageConsumer.IMAGEERROR||
			  status==ImageConsumer.IMAGEABORTED)
			error=true;
	}
	public void setDimensions(int width,int height)
	{
		w=width;
		h=height;
		pix=new int[w*h];
		scan=w;
	}
	public void setProperties(Hashtable props)
	{}
	public void setHints(int hints)
	{}
	public void setColorModel(ColorModel cm)
	{}
	public void setPixels(int startX,int startY,int wid, int ht,
		ColorModel cm,byte inPix[],int off,int scansize)
	{
		int cTable[]=new int[256];
		int i;
		for (i=0;i<256;i++)
			cTable[i]=cm.getRGB((byte)(128+(byte)i));
		int x,y;
		
		for (y=0;y<ht;y++)
		{
			int inDex=y*scansize+off;
			int outDex=(y+startY)*w+startX;
			for (x=0;x<wid;x++)
				pix[outDex+x]=cTable[inPix[inDex+x]];
		}
	}
	public void setPixels(int startX,int startY,int wid, int ht,
		ColorModel cm,int inPix[],int off,int scansize)
	{
		int x,y;
		
		for (y=0;y<ht;y++)
		{
			int inDex=y*scansize+off;
			int outDex=(y+startY)*w+startX;
			for (x=0;x<wid;x++)
				pix[outDex+x]=inPix[inDex+x];
		}
	}
/*External Interface*/
	public int init(Image img)
	{
		ImageProducer prod=img.getSource();
		prod.startProduction((ImageConsumer)this);
		while (!imageDone&&!error)
			{}//Wait for image to stream in.
		if (error)
			return 0;
		else 
			return 1;
	}
}
